/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla B2G.
 *
 * The Initial Developer of the Original Code is
 * Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

@namespace url("http://www.w3.org/1999/xhtml");
@namespace xul url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* Style the scrollbars */
xul|window xul|scrollbar {
  display: none;
}

html xul|scrollbar {
  display: block;
}

xul|scrollbar[orient="vertical"] {
  -moz-appearance: none !important;
  position: relative;
  margin-left: -8px;
  min-width: 8px;
  max-width: 8px;
  background-color: transparent !important;
  background-image: none !important;
  border: 0px solid transparent !important;
}

xul|scrollbar[orient="vertical"]:-moz-locale-dir(rtl) {
  margin-left: 2px;
  margin-right: -10px;
}

xul|scrollbar[orient="vertical"] xul|thumb {
  max-width: 6px !important;
  min-width: 6px !important;
}

xul|scrollbar[orient="horizontal"] {
  -moz-appearance: none !important;
  position: relative;
  min-height: 8px;
  max-height: 8px;
  margin-top: -8px;
  background-color: transparent !important;
  background-image: none !important;
  border: 0px solid transparent !important;
}

xul|scrollbar[orient="horizontal"] xul|thumb {
  max-height: 6px !important;
  min-height: 6px !important;
}

xul|scrollbox {
  overflow-y: scroll;
  overflow-x: scroll;
}

xul|scrollbarbutton {
  min-height: 8px !important;
  min-width: 8px !important;
  -moz-appearance: none !important;
  visibility: hidden;
}

xul|scrollbarbutton[sbattr="scrollbar-up-top"],
xul|scrollbarbutton[sbattr="scrollbar-bottom-top"] {
  display: none;
}

xul|thumb {
  background-color: rgba(0, 0, 0, 0.4) !important;
  -moz-border-top-colors: none !important;
  -moz-border-bottom-colors: none !important;
  -moz-border-right-colors: none !important;
  -moz-border-left-colors: none !important;
  border: 1px solid rgba(255, 255, 255, 0.4) !important;
  border-radius: 3px;
}

